import re
import nltk

from nltk.stem import WordNetLemmatizer
from nltk.tokenize import word_tokenize
import spacy

class TokensDict:
    def __init__(self):
        # Initialize NLTK
        nltk.download('punkt')
        nltk.download('wordnet')
        nltk.download('averaged_perceptron_tagger')
        nltk.download('omw-1.4')

    def tokenize_text(self, texts):
        tokens_list = []
        
        for text in texts:
            # Remove punctuation using regular expression
            prepro_text = re.sub(r'[^a-zA-Z0-9\s/-]', '', text)
            prepro_text = re.sub(r'[-/]', ' ', prepro_text)
            # Tokenization using NLTK
            tokens = word_tokenize(prepro_text)

            tokens_list.append(tokens)
        
        return tokens_list
    
    def remove_numbers(self, tokens_list):
        filtered_tokens_list = [[token for token in token_set if not token.isnumeric()] \
                                for token_set in tokens_list]

        return filtered_tokens_list

    def lemmatize_tokens(self, tokens_list):
        en = spacy.load('en_core_web_sm')
        stop_words = en.Defaults.stop_words

        # Initialize WordNet Lemmatizer
        lemmatizer = WordNetLemmatizer() 
        lemmatized_token_list = []

        for token_set in tokens_list:
            lemmatized_tokens = []
            tagged_tokens = nltk.pos_tag(token_set)

            for token, pos in tagged_tokens:
                if token not in stop_words:
                    if pos.startswith("N"):
                        lemmatized_tokens.append(lemmatizer.lemmatize(token, "n"))
                    elif pos.startswith("V"):
                        lemmatized_tokens.append(lemmatizer.lemmatize(token, "v"))
                    elif pos.startswith('J'):
                        lemmatized_tokens.append(lemmatizer.lemmatize(token, "a"))
                    elif pos.startswith('R'):
                        lemmatized_tokens.append(lemmatizer.lemmatize(token, "r"))
                    else:
                        lemmatized_tokens.append(token)

            lemmatized_token_list.append(lemmatized_tokens)
            
        return lemmatized_token_list
    
    def lowercase_tokens(self, tokens_list):
        return [[token.lower() for token in token_set] for token_set in tokens_list]
    
    def clean_text(self, text):
        cleaned_text = re.sub(r'\n', '', text)
        cleaned_text = re.sub(r'\s+', ' ', cleaned_text)
        cleaned_text = re.sub(r'^\[[^\]]*\]\s*', '', cleaned_text)

        return cleaned_text

    def clean_texts(self, texts):
        cleaned_texts = []

        for text in texts:
            cleaned_text = self.clean_text(text)

            cleaned_texts.append(cleaned_text)

        return cleaned_texts